use actix_web::error::ErrorInternalServerError;
use image::{imageops::FilterType, DynamicImage};
use std::{cmp::max, collections::HashMap, io::Cursor};

pub const THUMB_WIDTHS: [u32; 7] = [10, 40, 110, 300, 600, 800, 1400];

/// Парсит запрос на миниатюру, извлекая оригинальное имя файла и требуемую ширину.
/// Пример: "filename_150.ext" -> ("filename", 150, "ext")
/// unsafe/1440x/production/image/439efaa0-816f-11ef-b201-439da98539bc.jpg -> ("439efaa0-816f-11ef-b201-439da98539bc", 1440, "jpg")
pub fn parse_image_request(path: &str) -> Option<(String, u32, String)> {
    let mut path_parts = path.rsplit('/').collect::<Vec<&str>>();
    if let Some(filename_part) = path_parts.pop() {
        let mut oldwidth = 0;
        if path.starts_with("unsafe") {
            if let Some(old_width_str) = path_parts.get(1) {
            let mut old_width_str = old_width_str.to_string();
            if old_width_str.ends_with('x') {
                old_width_str.pop();
                if let Ok(width) = old_width_str.parse::<u32>() {
                    oldwidth = width;
                    }
                }
            }
        }
        if let Some((name_part, ext_part)) = filename_part.rsplit_once('.') {
            if let Some((base_name, width_str)) = name_part.rsplit_once('_') {
                if let Ok(width) = width_str.parse::<u32>() {
                    return Some((base_name.to_string(), max(width, oldwidth), ext_part.to_string()));
                }
            }
            return Some((name_part.to_string(), 0, ext_part.to_string().to_lowercase()));
        }
    }
    None
}

/// Выбирает ближайший подходящий размер из предопределённых.
pub fn find_closest_width(requested_width: u32) -> u32 {
    *THUMB_WIDTHS
        .iter()
        .min_by_key(|&&width| (width as i32 - requested_width as i32).abs())
        .unwrap_or(&THUMB_WIDTHS[0]) // Возвращаем самый маленький размер, если ничего не подошло
}

/// Генерирует миниатюры изображения.
pub async fn generate_thumbnails(image: &DynamicImage) -> Result<HashMap<u32, Vec<u8>>, actix_web::Error> {
    let mut thumbnails = HashMap::new();

    for &width in THUMB_WIDTHS.iter().filter(|&&w| w < image.width()) {
        let thumbnail = image.resize(width, u32::MAX, FilterType::Lanczos3); // Ресайз изображения по ширине
        let mut buffer = Vec::new();
        thumbnail
            .write_to(&mut Cursor::new(&mut buffer), image::ImageFormat::Jpeg)
            .map_err(|_| ErrorInternalServerError("Failed to generate thumbnail"))?; // Сохранение изображения в формате JPEG
        thumbnails.insert(width, buffer);
    }

    Ok(thumbnails)
}
